# -*- coding: utf-8 -*-

import os, platform, time

from .utils import (_fetch_txt_files, 
                    _collect_errors, 
                    _create_errors_table,
                    _write_detail_summary,
                    _write_agg_summary)

if platform.system() == "Darwin" or platform.system() == "Linux":
    
    TEMP_FOLDER = "/tmp"

elif platform.system() == "Windows":

    TEMP_FOLDER = os.environ["TEMP"]
    

def main(input_dir, output_dir, output_file, queue, **kwargs):
    
    start_time = time.time()
        
    input_files = _fetch_txt_files(input_dir)
    
    if output_file == "":
        output_file = "error_results.csv"
        
    db_file = os.path.splitext(output_file)[0] + ".sqlite"
            
    output_file = os.path.join(output_dir, output_file)
    db_file = os.path.join(TEMP_FOLDER, db_file)
        
    attempted_files = 0 # Track how many files attempted processing
    completed_files = 0 # Track how many files were processed successfully
    bad_input_files = list() # Store any files with invalid characters
    good_input_files = list()
        
    # Create empty list to store database rows
    global_errors = list()
    
    
    # Main loop that batch processes input files
    for i, input_file in enumerate(input_files):
        
        attempted_files += 1

        # Collect errors from current text file
        curr_errors = _collect_errors(input_file)
        # Return of None indicates error in reading file
        if curr_errors is None:
            # Add to list of bad input files
            bad_input_files.append(input_file)
            # Skip to processing the next file
            continue
        
        completed_files += 1
        good_input_files.append(input_file)
        
        if int(100 * (float(completed_files) / len(input_files))) % 10 == 0:
            queue.put("Processed {0:d}% of files successfully ...".format(
                    int(100 * (float(completed_files) / len(input_files)))))
     
        # Add current errors to the global error log
        global_errors.extend(curr_errors)
    
    # Filter out style and uncategorized error first
    global_errors = [error for error in global_errors if kwargs.get(error.category)]
    
    # Then filter out errors that the user didn't request
    global_errors = [error for error in global_errors if kwargs.get(error.category).get()]
    
    # Sort global errors by error category, directory, then file
    global_errors = sorted(global_errors, key=lambda x: (x.category, x.filename))
    
    if kwargs.get("detail").get():
        output_name, output_ext = os.path.splitext(output_file)
        detail_file = "{0:s}_detail{1:s}".format(output_name, output_ext)
        _write_detail_summary(global_errors, detail_file)
    
    del kwargs["detail"]

    try:
        # Write rows to local SQLite database
        _create_errors_table(global_errors, db_file)
        _write_agg_summary(good_input_files, output_file, db_file, **kwargs)
        queue.put("{0:d} of {1:d} files processed in {2:.3f} minutes".format(
            completed_files, attempted_files, (time.time() - start_time) / 60))
    except Exception:
        raise
    finally:
        os.remove(db_file)
    