# -*- coding: utf-8 -*-

import csv, glob, itertools, os, platform, re, sys, sqlite3
from collections import namedtuple

# 3rd-party imports
import language_check

from .all_errors import ALL_ERRORS

# Initialize grammar and spelling checker for US English
ERROR_TOOL = language_check.LanguageTool('en-US')

# Make row template with column names for each error dimension
ERROR = namedtuple('Error', field_names=
                   ['context', 'rule', 'msg', 'category', 'filename'])

if platform.system() == "Darwin" or platform.system() == "Linux":
    
    TEMP_FOLDER = "/tmp"

elif platform.system() == "Windows":

    TEMP_FOLDER = os.environ["TEMP"]


def _read_input(input_file):
    
    # Read file in binary mode so all ASCII and non-ASCII characters are found
    with open(input_file, 'rb') as content_handle:
            content = content_handle.read()
    return content

        
def _clean_open(input_file):
    
    content = _read_input(input_file)
    
    try:
        # Replace known non-ASCII characters with ASCII characters
        cleaned_content = \
            content\
            .replace(b'&nbsp;', b'')\
            .replace(b'\xa0', b'')\
            .replace(b'\xc2', b'')\
            .replace(b'\xe9', b'e')\
            .replace(b'\x85', b'')\
            .replace(b'\x91', b"'")\
            .replace(b'\x92', b"'")\
            .replace(b'\x93', b'"')\
            .replace(b'\x94', b'"')\
            .replace(b'\x96', b'-')\
            .decode() # Convert from bytes to string
        
    except UnicodeDecodeError:
        raise
    
    # Clean-up unformatted quotation marks
    cleaned_content = re.sub(r'&?quot\b;*', "''", cleaned_content)
            
    # Remove all tabs, new lines, and carriage returns
    cleaned_content = ' '.join(cleaned_content.split())
    
    return cleaned_content
            
        
def _fetch_txt_files(input_path):
    
    """Recursively finds and returns list of all .txt files in a path"""
    
    # Create list to store .txt files
    files = glob.glob(input_path + '/*.txt')
    # Loop through all paths (sub-dirs and files) in input_path
    for sub_path in glob.iglob(input_path + '/*'):
        # If .txt files are found in a path -- 
        if glob.iglob(sub_path + '/*.txt'):
            # -- Then add them to the files list
            files.extend(_fetch_txt_files(sub_path))
        # Recursively search each sub-path for more .txt files
    # Send .txt files up the recursive stack
    return files


def _collect_errors(input_file, **kwargs):
    
    """Finds grammar and spelling errors in a text file, extracts relevant
    dimensions about the errors, and formats them for database insertion
    """
    # Make list to store table rows
    
    try:
        # Open current file and read in text content        
        content = _clean_open(input_file)
        # Extract grammar and spelling errors from the text content
        errors = ERROR_TOOL.check(content)
    # If invalid Unicode is encountered --
    except UnicodeDecodeError:
        return
    
    return [ERROR(context=error.context,
                        rule=error.ruleId,
                        msg=error.msg,
                        category=error.locqualityissuetype,
                        filename=input_file)
                 for error in errors]
            

def _create_errors_table(rows, db_file):
    
    """Creates a local SQLite database and an errors table with rows 
    containing info on grammar and spelling errors found in a batch
    of text files
    """
    
    # Create and connect to a local errors database
    conn = sqlite3.connect(db_file)
    # Start a cursor to begin executing commands
    c = conn.cursor()
    
    # Within the database, create the main table to hold dimemsions on the errors
    c.execute('CREATE TABLE errors'\
              ' (context TEXT, rule TEXT, msg TEXT, category TEXT,'\
              ' filename TEXT)')
    # Within the table, add the category, rule, context, and file of each error
    c.executemany("INSERT INTO errors"\
                  " (context, rule, msg, category, filename) VALUES"\
                  " (?, ?, ?, ?, ?)", rows)
    
    # Commit all insertions that were executed on the database
    conn.commit()
    # Close database connection
    conn.close()
    
    
def _write_detail_summary(errors, detail_file):
    
    fieldnames = ['context', 'rule', 'msg', 'category', 'filename']
    # Combine error dimensions with column labels to form rows
    rows = [dict(zip(fieldnames, row)) for row in errors]
        
    # Open file in regular write mode if Python 3.x
    mode = 'w'
    # Open file in binary write mode if Python 2.7
    if sys.version_info.major < 3:
        mode += 'b'
        # Also encode error dimensions for each row in UTF-8 for csv writing
        for row in rows:
            for key, value in row.iteritems():
                # Only encode non-null values
                if value:
                    row[key] = value.encode('utf-8')
    # Write error summary table to local csv file
    with open(detail_file, mode) as output_handle:
        writer = csv.DictWriter(output_handle, fieldnames=fieldnames)
        writer.writeheader()
        writer.writerows(rows)
        
        
def _write_agg_summary(input_files, output_file, db_file, **kwargs):
    
    """Writes aggregate summary table to local CSV file, containing counts for 
    each file's error categories, subcategories, and rules
    """
    
    # Connect to the local database
    conn = sqlite3.connect(os.path.join(TEMP_FOLDER, db_file))
    # Start a cursor to begin executing commands
    c = conn.cursor()
    
    fieldnames = ["filename", "error_count", "word_count"]
    categories = {k:v for k,v in kwargs.iteritems() if v.get()}
    categories = sorted(categories.keys())
    
    corpus_rules = set(itertools.chain.from_iterable(c.execute(
            "SELECT DISTINCT rule FROM errors")))
    all_rules = sorted(list(corpus_rules | ALL_ERRORS))
    
    fieldnames = fieldnames + categories + all_rules
    
    # Create row list to hold error dimensions for each text in input directory
    agg_rows = list()
    
    # Loop through each file
    for input_file in input_files:
        # Count number of words in document for normalizing error count
        word_count = len(_clean_open(input_file).split())
        # Create dictionary to represent a single table row 
        dim_counts = dict(filename=input_file)
        # Start count of errors 
        error_count = 0
        # Count the occurrences of each error category 
        c.execute("SELECT category, COUNT(*) FROM errors WHERE"\
                  " filename = ? GROUP BY category", (input_file, ))
        found_categories = dict(c.fetchall())
        
        # Go through each category
        for category in categories:
            # Add each category count to row dictionary
            # Set non-occurring categories to 0
            dim_counts[category] = found_categories.get(category, 0)
            # Add each category count to total error count
            error_count += dim_counts[category]
            
        # Add total count to row dictionary
        dim_counts['error_count'] = error_count
        # Add word count to row dictionary
        dim_counts['word_count'] = word_count
            
        # Count the occurrences of each error rule
        c.execute("SELECT rule, COUNT(*) FROM errors WHERE"\
                  " filename = ? GROUP BY rule", (input_file, ))
        found_rules = dict(c.fetchall())
        
        # Add occurrences to row dictionary, set non-occurring error rules to 0
        for rule in all_rules:
            dim_counts[rule] = found_rules.get(rule, 0)
    
        # Add row dictionary to row list
        agg_rows.append(dim_counts)
            
    # Close database connection
    conn.close()
    
    # Open file in regular write mode if Python 3.x
    mode = 'w'
    # Open file in binary write mode if Python 2.7
    if sys.version_info.major < 3:
        mode += 'b'
    # Write summary table to local csv file
    with open(output_file, mode) as output_handle:
        writer = csv.DictWriter(output_handle, fieldnames=fieldnames)
        writer.writeheader()
        writer.writerows(agg_rows)
    